@@ -231,17 +231,19 @@ impl InsertStatement {
                 val_len: values.len(),
             });
         }
-        let values_source = if let Some(InsertValueSource::Values(values)) = &mut self.source {
-            values
-        } else {
-            self.source = Some(InsertValueSource::Values(Default::default()));
-            if let Some(InsertValueSource::Values(values)) = &mut self.source {
+        if !values.is_empty() {
+            let values_source = if let Some(InsertValueSource::Values(values)) = &mut self.source {
                 values
             } else {
-                unreachable!();
-            }
-        };
-        values_source.push(values);
+                self.source = Some(InsertValueSource::Values(Default::default()));
+                if let Some(InsertValueSource::Values(values)) = &mut self.source {
+                    values
+                } else {
+                    unreachable!();
+                }
+            };
+            values_source.push(values);
+        }
         Ok(self)
     }
 
