@@ -15,6 +15,7 @@ mod select;
 mod shim;
 mod traits;
 mod update;
+mod with;
 
 pub use condition::*;
 pub use delete::*;
@@ -23,6 +24,7 @@ pub use ordered::*;
 pub use select::*;
 pub use traits::*;
 pub use update::*;
+pub use with::*;
 
 /// Shorthand for constructing any table query
 #[derive(Debug, Clone)]
@@ -37,6 +39,15 @@ pub enum QueryStatement {
     Delete(DeleteStatement),
 }
 
+#[derive(Debug, Clone)]
+pub enum SubQueryStatement {
+    SelectStatement(SelectStatement),
+    InsertStatement(InsertStatement),
+    UpdateStatement(UpdateStatement),
+    DeleteStatement(DeleteStatement),
+    WithStatement(WithQuery),
+}
+
 impl Query {
     /// Construct table [`SelectStatement`]
     pub fn select() -> SelectStatement {
@@ -57,4 +68,9 @@ impl Query {
     pub fn delete() -> DeleteStatement {
         DeleteStatement::new()
     }
+
+    /// Construct [`WithClause`]
+    pub fn with() -> WithClause {
+        WithClause::new()
+    }
 }
