@@ -3,13 +3,14 @@ use crate::{types::*, value::*};
 /// Specification of a table column
 #[derive(Debug, Clone)]
 pub struct ColumnDef {
-    pub(crate) table: Option<DynIden>,
+    pub(crate) table: Option<TableRef>,
     pub(crate) name: DynIden,
     pub(crate) types: Option<ColumnType>,
     pub(crate) spec: Vec<ColumnSpec>,
 }
 
 /// All column types
+#[non_exhaustive]
 #[derive(Debug, Clone)]
 pub enum ColumnType {
     Char(Option<u32>),
@@ -27,7 +28,7 @@ pub enum ColumnType {
     TimestampWithTimeZone(Option<u32>),
     Time(Option<u32>),
     Date,
-    Interval(Option<IntervalField>, Option<u32>),
+    Interval(Option<PgInterval>, Option<u32>),
     Binary(Option<u32>),
     Boolean,
     Money(Option<(u32, u32)>),
@@ -35,6 +36,8 @@ pub enum ColumnType {
     JsonBinary,
     Uuid,
     Custom(DynIden),
+    Enum(String, Vec<String>),
+    Array(Option<String>),
 }
 
 /// All column specification keywords
@@ -51,7 +54,7 @@ pub enum ColumnSpec {
 
 // All interval fields
 #[derive(Debug, Clone)]
-pub enum IntervalField {
+pub enum PgInterval {
     Year,
     Month,
     Day,
@@ -67,6 +70,28 @@ pub enum IntervalField {
     MinuteToSecond,
 }
 
+#[cfg(feature = "postgres-interval")]
+impl quote::ToTokens for PgInterval {
+    fn to_tokens(&self, tokens: &mut proc_macro2::TokenStream) {
+        use quote::{quote, TokenStreamExt};
+        tokens.append_all(match self {
+            PgInterval::Year => quote! { PgInterval::Year },
+            PgInterval::Month => quote! { PgInterval::Month },
+            PgInterval::Day => quote! { PgInterval::Day },
+            PgInterval::Hour => quote! { PgInterval::Hour },
+            PgInterval::Minute => quote! { PgInterval::Minute },
+            PgInterval::Second => quote! { PgInterval::Second },
+            PgInterval::YearToMonth => quote! { PgInterval::YearToMonth },
+            PgInterval::DayToHour => quote! { PgInterval::DayToHour },
+            PgInterval::DayToMinute => quote! { PgInterval::DayToMinute },
+            PgInterval::DayToSecond => quote! { PgInterval::DayToSecond },
+            PgInterval::HourToMinute => quote! { PgInterval::HourToMinute },
+            PgInterval::HourToSecond => quote! { PgInterval::HourToSecond },
+            PgInterval::MinuteToSecond => quote! { PgInterval::MinuteToSecond },
+        });
+    }
+}
+
 impl ColumnDef {
     /// Construct a table column
     pub fn new<T: 'static>(name: T) -> Self
@@ -267,7 +292,7 @@ impl ColumnDef {
     ///         )
     ///         .col(
     ///             ColumnDef::new(Alias::new("I2"))
-    ///                 .interval(Some(IntervalField::YearToMonth), None)
+    ///                 .interval(Some(PgInterval::YearToMonth), None)
     ///                 .not_null()
     ///         )
     ///         .col(
@@ -277,7 +302,7 @@ impl ColumnDef {
     ///         )
     ///         .col(
     ///             ColumnDef::new(Alias::new("I4"))
-    ///                 .interval(Some(IntervalField::Hour), Some(43))
+    ///                 .interval(Some(PgInterval::Hour), Some(43))
     ///                 .not_null()
     ///         )
     ///         .to_string(PostgresQueryBuilder),
@@ -293,7 +318,7 @@ impl ColumnDef {
     /// );
     /// ```
     #[cfg(feature = "backend-postgres")]
-    pub fn interval(&mut self, fields: Option<IntervalField>, precision: Option<u32>) -> &mut Self {
+    pub fn interval(&mut self, fields: Option<PgInterval>, precision: Option<u32>) -> &mut Self {
         self.types = Some(ColumnType::Interval(fields, precision));
         self
     }
@@ -401,6 +426,27 @@ impl ColumnDef {
         self
     }
 
+    /// Set column type as enum.
+    pub fn enumeration<N, S, V>(&mut self, name: N, variants: V) -> &mut Self
+    where
+        N: ToString,
+        S: ToString,
+        V: IntoIterator<Item = S>,
+    {
+        self.types = Some(ColumnType::Enum(
+            name.to_string(),
+            variants.into_iter().map(|v| v.to_string()).collect(),
+        ));
+        self
+    }
+
+    /// Set column type as an array with a specified element type.
+    /// This is only supported on Postgres.
+    pub fn array(&mut self, elem_type: String) -> &mut Self {
+        self.types = Some(ColumnType::Array(Some(elem_type)));
+        self
+    }
+
     /// Some extra options in custom string
     pub fn extra(&mut self, string: String) -> &mut Self {
         self.spec.push(ColumnSpec::Extra(string));
