@@ -7,7 +7,7 @@ use serde_json::Value as Json;
 use std::str::from_utf8;
 
 #[cfg(feature = "with-chrono")]
-use chrono::{DateTime, FixedOffset, NaiveDate, NaiveDateTime, NaiveTime};
+use chrono::{DateTime, FixedOffset, Local, NaiveDate, NaiveDateTime, NaiveTime, Utc};
 
 #[cfg(feature = "with-rust_decimal")]
 use rust_decimal::Decimal;
@@ -38,7 +38,7 @@ pub enum Value {
     Double(Option<f64>),
     String(Option<Box<String>>),
 
-    #[allow(clippy::box_vec)]
+    #[allow(clippy::box_collection)]
     Bytes(Option<Box<Vec<u8>>>),
 
     #[cfg(feature = "with-json")]
@@ -57,6 +57,14 @@ pub enum Value {
     #[cfg_attr(docsrs, doc(cfg(feature = "with-chrono")))]
     DateTime(Option<Box<NaiveDateTime>>),
 
+    #[cfg(feature = "with-chrono")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "with-chrono")))]
+    DateTimeUtc(Option<Box<DateTime<Utc>>>),
+
+    #[cfg(feature = "with-chrono")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "with-chrono")))]
+    DateTimeLocal(Option<Box<DateTime<Local>>>),
+
     #[cfg(feature = "with-chrono")]
     #[cfg_attr(docsrs, doc(cfg(feature = "with-chrono")))]
     DateTimeWithTimeZone(Option<Box<DateTime<FixedOffset>>>),
@@ -72,6 +80,10 @@ pub enum Value {
     #[cfg(feature = "with-bigdecimal")]
     #[cfg_attr(docsrs, doc(cfg(feature = "with-bigdecimal")))]
     BigDecimal(Option<Box<BigDecimal>>),
+
+    #[cfg(feature = "postgres-array")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "postgres-array")))]
+    Array(Option<Box<Vec<Value>>>),
 }
 
 pub trait ValueType: Sized {
@@ -105,6 +117,9 @@ pub enum ValueTuple {
     One(Value),
     Two(Value, Value),
     Three(Value, Value, Value),
+    Four(Value, Value, Value, Value),
+    Five(Value, Value, Value, Value, Value),
+    Six(Value, Value, Value, Value, Value, Value),
 }
 
 pub trait IntoValueTuple {
@@ -279,22 +294,77 @@ mod with_json {
 #[cfg_attr(docsrs, doc(cfg(feature = "with-chrono")))]
 mod with_chrono {
     use super::*;
-    use chrono::{Offset, TimeZone};
+    use chrono::{Local, Offset, Utc};
 
     type_to_box_value!(NaiveDate, Date, Date);
     type_to_box_value!(NaiveTime, Time, Time(None));
     type_to_box_value!(NaiveDateTime, DateTime, DateTime(None));
 
-    impl<Tz> From<DateTime<Tz>> for Value
-    where
-        Tz: TimeZone,
-    {
-        fn from(x: DateTime<Tz>) -> Value {
+    impl From<DateTime<Utc>> for Value {
+        fn from(v: DateTime<Utc>) -> Value {
+            Value::DateTimeUtc(Some(Box::new(v)))
+        }
+    }
+
+    impl From<DateTime<Local>> for Value {
+        fn from(v: DateTime<Local>) -> Value {
+            Value::DateTimeLocal(Some(Box::new(v)))
+        }
+    }
+
+    impl From<DateTime<FixedOffset>> for Value {
+        fn from(x: DateTime<FixedOffset>) -> Value {
             let v = DateTime::<FixedOffset>::from_utc(x.naive_utc(), x.offset().fix());
             Value::DateTimeWithTimeZone(Some(Box::new(v)))
         }
     }
 
+    impl Nullable for DateTime<Utc> {
+        fn null() -> Value {
+            Value::DateTimeUtc(None)
+        }
+    }
+
+    impl ValueType for DateTime<Utc> {
+        fn try_from(v: Value) -> Result<Self, ValueTypeErr> {
+            match v {
+                Value::DateTimeUtc(Some(x)) => Ok(*x),
+                _ => Err(ValueTypeErr),
+            }
+        }
+
+        fn type_name() -> String {
+            stringify!(DateTime<Utc>).to_owned()
+        }
+
+        fn column_type() -> ColumnType {
+            ColumnType::TimestampWithTimeZone(None)
+        }
+    }
+
+    impl Nullable for DateTime<Local> {
+        fn null() -> Value {
+            Value::DateTimeLocal(None)
+        }
+    }
+
+    impl ValueType for DateTime<Local> {
+        fn try_from(v: Value) -> Result<Self, ValueTypeErr> {
+            match v {
+                Value::DateTimeLocal(Some(x)) => Ok(*x),
+                _ => Err(ValueTypeErr),
+            }
+        }
+
+        fn type_name() -> String {
+            stringify!(DateTime<Local>).to_owned()
+        }
+
+        fn column_type() -> ColumnType {
+            ColumnType::TimestampWithTimeZone(None)
+        }
+    }
+
     impl Nullable for DateTime<FixedOffset> {
         fn null() -> Value {
             Value::DateTimeWithTimeZone(None)
@@ -343,6 +413,92 @@ mod with_uuid {
     type_to_box_value!(Uuid, Uuid, Uuid);
 }
 
+#[cfg(feature = "postgres-array")]
+#[cfg_attr(docsrs, doc(cfg(feature = "postgres-array")))]
+mod with_array {
+    use super::*;
+
+    // We only imlement conversion from Vec<T> to Array when T is not u8.
+    // This is because for u8's case, there is already conversion to Byte defined above.
+    // TODO When negative trait becomes a stable feature, following code can be much shorter.
+    pub trait NotU8 {}
+
+    impl NotU8 for bool {}
+    impl NotU8 for i8 {}
+    impl NotU8 for i16 {}
+    impl NotU8 for i32 {}
+    impl NotU8 for i64 {}
+    impl NotU8 for u16 {}
+    impl NotU8 for u32 {}
+    impl NotU8 for u64 {}
+    impl NotU8 for f32 {}
+    impl NotU8 for f64 {}
+    impl NotU8 for String {}
+
+    #[cfg(feature = "with-json")]
+    impl NotU8 for Json {}
+
+    #[cfg(feature = "with-chrono")]
+    impl NotU8 for NaiveDate {}
+
+    #[cfg(feature = "with-chrono")]
+    impl NotU8 for NaiveTime {}
+
+    #[cfg(feature = "with-chrono")]
+    impl NotU8 for NaiveDateTime {}
+
+    #[cfg(feature = "with-chrono")]
+    impl<Tz> NotU8 for DateTime<Tz> where Tz: chrono::TimeZone {}
+
+    #[cfg(feature = "with-rust_decimal")]
+    impl NotU8 for Decimal {}
+
+    #[cfg(feature = "with-bigdecimal")]
+    impl NotU8 for BigDecimal {}
+
+    #[cfg(feature = "with-uuid")]
+    impl NotU8 for Uuid {}
+
+    impl<T> From<Vec<T>> for Value
+    where
+        T: Into<Value> + NotU8,
+    {
+        fn from(x: Vec<T>) -> Value {
+            Value::Array(Some(Box::new(x.into_iter().map(|e| e.into()).collect())))
+        }
+    }
+
+    impl<T> Nullable for Vec<T>
+    where
+        T: Into<Value> + NotU8,
+    {
+        fn null() -> Value {
+            Value::Array(None)
+        }
+    }
+
+    impl<T> ValueType for Vec<T>
+    where
+        T: NotU8 + ValueType,
+    {
+        fn try_from(v: Value) -> Result<Self, ValueTypeErr> {
+            match v {
+                Value::Array(Some(v)) => Ok(v.into_iter().map(|e| e.unwrap()).collect()),
+                _ => Err(ValueTypeErr),
+            }
+        }
+
+        fn type_name() -> String {
+            stringify!(Vec<T>).to_owned()
+        }
+
+        fn column_type() -> ColumnType {
+            use ColumnType::*;
+            Array(None)
+        }
+    }
+}
+
 #[allow(unused_macros)]
 macro_rules! box_to_opt_ref {
     ( $v: expr ) => {
@@ -434,12 +590,50 @@ impl Value {
 }
 
 impl Value {
+    pub fn is_date_time_utc(&self) -> bool {
+        #[cfg(feature = "with-chrono")]
+        return matches!(self, Self::DateTimeUtc(_));
+        #[cfg(not(feature = "with-chrono"))]
+        return false;
+    }
+
+    pub fn is_date_time_local(&self) -> bool {
+        #[cfg(feature = "with-chrono")]
+        return matches!(self, Self::DateTimeLocal(_));
+        #[cfg(not(feature = "with-chrono"))]
+        return false;
+    }
     pub fn is_date_time_with_time_zone(&self) -> bool {
         #[cfg(feature = "with-chrono")]
         return matches!(self, Self::DateTimeWithTimeZone(_));
         #[cfg(not(feature = "with-chrono"))]
         return false;
     }
+
+    #[cfg(feature = "with-chrono")]
+    pub fn as_ref_date_time_utc(&self) -> Option<&DateTime<Utc>> {
+        match self {
+            Self::DateTimeUtc(v) => box_to_opt_ref!(v),
+            _ => panic!("not Value::DateTimeUtc"),
+        }
+    }
+    #[cfg(not(feature = "with-chrono"))]
+    pub fn as_ref_date_time_utc(&self) -> Option<&bool> {
+        panic!("not Value::DateTimeUtc")
+    }
+
+    #[cfg(feature = "with-chrono")]
+    pub fn as_ref_date_time_local(&self) -> Option<&DateTime<Local>> {
+        match self {
+            Self::DateTimeLocal(v) => box_to_opt_ref!(v),
+            _ => panic!("not Value::DateTimeLocal"),
+        }
+    }
+    #[cfg(not(feature = "with-chrono"))]
+    pub fn as_ref_date_time_local(&self) -> Option<&bool> {
+        panic!("not Value::DateTimeLocal")
+    }
+
     #[cfg(feature = "with-chrono")]
     pub fn as_ref_date_time_with_time_zone(&self) -> Option<&DateTime<FixedOffset>> {
         match self {
@@ -451,6 +645,21 @@ impl Value {
     pub fn as_ref_date_time_with_time_zone(&self) -> Option<&bool> {
         panic!("not Value::DateTimeWithTimeZone")
     }
+
+    #[cfg(feature = "with-chrono")]
+    pub fn as_naive_utc_in_string(&self) -> Option<String> {
+        match self {
+            Self::DateTime(v) => v.as_ref().map(|v| v.to_string()),
+            Self::DateTimeUtc(v) => v.as_ref().map(|v| v.naive_utc().to_string()),
+            Self::DateTimeLocal(v) => v.as_ref().map(|v| v.naive_utc().to_string()),
+            Self::DateTimeWithTimeZone(v) => v.as_ref().map(|v| v.naive_utc().to_string()),
+            _ => panic!("not Value::DateTime"),
+        }
+    }
+    #[cfg(not(feature = "with-chrono"))]
+    pub fn as_naive_utc_in_string(&self) -> Option<&bool> {
+        panic!("not Value::DateTime")
+    }
 }
 
 impl Value {
@@ -531,6 +740,27 @@ impl Value {
     }
 }
 
+impl Value {
+    pub fn is_array(&self) -> bool {
+        #[cfg(feature = "postgres-array")]
+        return matches!(self, Self::Array(_));
+        #[cfg(not(feature = "postgres-array"))]
+        return false;
+    }
+
+    #[cfg(feature = "postgres-array")]
+    pub fn as_ref_array(&self) -> Option<&Vec<Value>> {
+        match self {
+            Self::Array(v) => box_to_opt_ref!(v),
+            _ => panic!("not Value::Array"),
+        }
+    }
+    #[cfg(not(feature = "postgres-array"))]
+    pub fn as_ref_array(&self) -> Option<&bool> {
+        panic!("not Value::Array")
+    }
+}
+
 impl IntoIterator for ValueTuple {
     type Item = Value;
     type IntoIter = std::vec::IntoIter<Self::Item>;
@@ -540,6 +770,9 @@ impl IntoIterator for ValueTuple {
             ValueTuple::One(v) => vec![v].into_iter(),
             ValueTuple::Two(v, w) => vec![v, w].into_iter(),
             ValueTuple::Three(u, v, w) => vec![u, v, w].into_iter(),
+            ValueTuple::Four(u, v, w, x) => vec![u, v, w, x].into_iter(),
+            ValueTuple::Five(u, v, w, x, y) => vec![u, v, w, x, y].into_iter(),
+            ValueTuple::Six(u, v, w, x, y, z) => vec![u, v, w, x, y, z].into_iter(),
         }
     }
 }
@@ -580,6 +813,58 @@ where
     }
 }
 
+impl<U, V, W, X> IntoValueTuple for (U, V, W, X)
+where
+    U: Into<Value>,
+    V: Into<Value>,
+    W: Into<Value>,
+    X: Into<Value>,
+{
+    fn into_value_tuple(self) -> ValueTuple {
+        ValueTuple::Four(self.0.into(), self.1.into(), self.2.into(), self.3.into())
+    }
+}
+
+impl<U, V, W, X, Y> IntoValueTuple for (U, V, W, X, Y)
+where
+    U: Into<Value>,
+    V: Into<Value>,
+    W: Into<Value>,
+    X: Into<Value>,
+    Y: Into<Value>,
+{
+    fn into_value_tuple(self) -> ValueTuple {
+        ValueTuple::Five(
+            self.0.into(),
+            self.1.into(),
+            self.2.into(),
+            self.3.into(),
+            self.4.into(),
+        )
+    }
+}
+
+impl<U, V, W, X, Y, Z> IntoValueTuple for (U, V, W, X, Y, Z)
+where
+    U: Into<Value>,
+    V: Into<Value>,
+    W: Into<Value>,
+    X: Into<Value>,
+    Y: Into<Value>,
+    Z: Into<Value>,
+{
+    fn into_value_tuple(self) -> ValueTuple {
+        ValueTuple::Six(
+            self.0.into(),
+            self.1.into(),
+            self.2.into(),
+            self.3.into(),
+            self.4.into(),
+            self.5.into(),
+        )
+    }
+}
+
 impl<V> FromValueTuple for V
 where
     V: Into<Value> + ValueType,
@@ -628,6 +913,72 @@ where
     }
 }
 
+impl<U, V, W, X> FromValueTuple for (U, V, W, X)
+where
+    U: Into<Value> + ValueType,
+    V: Into<Value> + ValueType,
+    W: Into<Value> + ValueType,
+    X: Into<Value> + ValueType,
+{
+    fn from_value_tuple<I>(i: I) -> Self
+    where
+        I: IntoValueTuple,
+    {
+        match i.into_value_tuple() {
+            ValueTuple::Four(u, v, w, x) => (u.unwrap(), v.unwrap(), w.unwrap(), x.unwrap()),
+            _ => panic!("not ValueTuple::Four"),
+        }
+    }
+}
+
+impl<U, V, W, X, Y> FromValueTuple for (U, V, W, X, Y)
+where
+    U: Into<Value> + ValueType,
+    V: Into<Value> + ValueType,
+    W: Into<Value> + ValueType,
+    X: Into<Value> + ValueType,
+    Y: Into<Value> + ValueType,
+{
+    fn from_value_tuple<I>(i: I) -> Self
+    where
+        I: IntoValueTuple,
+    {
+        match i.into_value_tuple() {
+            ValueTuple::Five(u, v, w, x, y) => {
+                (u.unwrap(), v.unwrap(), w.unwrap(), x.unwrap(), y.unwrap())
+            }
+            _ => panic!("not ValueTuple::Five"),
+        }
+    }
+}
+
+impl<U, V, W, X, Y, Z> FromValueTuple for (U, V, W, X, Y, Z)
+where
+    U: Into<Value> + ValueType,
+    V: Into<Value> + ValueType,
+    W: Into<Value> + ValueType,
+    X: Into<Value> + ValueType,
+    Y: Into<Value> + ValueType,
+    Z: Into<Value> + ValueType,
+{
+    fn from_value_tuple<I>(i: I) -> Self
+    where
+        I: IntoValueTuple,
+    {
+        match i.into_value_tuple() {
+            ValueTuple::Six(u, v, w, x, y, z) => (
+                u.unwrap(),
+                v.unwrap(),
+                w.unwrap(),
+                x.unwrap(),
+                y.unwrap(),
+                z.unwrap(),
+            ),
+            _ => panic!("not ValueTuple::Six"),
+        }
+    }
+}
+
 /// Escape a SQL string literal
 pub fn escape_string(string: &str) -> String {
     string
@@ -700,6 +1051,8 @@ pub fn sea_value_to_json_value(value: &Value) -> Json {
         Value::BigDecimal(None) => Json::Null,
         #[cfg(feature = "with-uuid")]
         Value::Uuid(None) => Json::Null,
+        #[cfg(feature = "postgres-array")]
+        Value::Array(None) => Json::Null,
         Value::Bool(Some(b)) => Json::Bool(*b),
         Value::TinyInt(Some(v)) => (*v).into(),
         Value::SmallInt(Some(v)) => (*v).into(),
@@ -722,6 +1075,10 @@ pub fn sea_value_to_json_value(value: &Value) -> Json {
         Value::DateTime(_) => CommonSqlQueryBuilder.value_to_string(value).into(),
         #[cfg(feature = "with-chrono")]
         Value::DateTimeWithTimeZone(_) => CommonSqlQueryBuilder.value_to_string(value).into(),
+        #[cfg(feature = "with-chrono")]
+        Value::DateTimeUtc(_) => CommonSqlQueryBuilder.value_to_string(value).into(),
+        #[cfg(feature = "with-chrono")]
+        Value::DateTimeLocal(_) => CommonSqlQueryBuilder.value_to_string(value).into(),
         #[cfg(feature = "with-rust_decimal")]
         Value::Decimal(Some(v)) => {
             use rust_decimal::prelude::ToPrimitive;
@@ -734,6 +1091,13 @@ pub fn sea_value_to_json_value(value: &Value) -> Json {
         }
         #[cfg(feature = "with-uuid")]
         Value::Uuid(Some(v)) => Json::String(v.to_string()),
+        #[cfg(feature = "postgres-array")]
+        Value::Array(Some(v)) => Json::Array(
+            v.as_ref()
+                .iter()
+                .map(|v| sea_value_to_json_value(v))
+                .collect(),
+        ),
     }
 }
 
@@ -861,6 +1225,36 @@ mod tests {
                 Value::String(Some(Box::new("b".to_owned())))
             )
         );
+        assert_eq!(
+            (1i32, 2.4f64, "b", 123u8).into_value_tuple(),
+            ValueTuple::Four(
+                Value::Int(Some(1)),
+                Value::Double(Some(2.4)),
+                Value::String(Some(Box::new("b".to_owned()))),
+                Value::TinyUnsigned(Some(123))
+            )
+        );
+        assert_eq!(
+            (1i32, 2.4f64, "b", 123u8, 456u16).into_value_tuple(),
+            ValueTuple::Five(
+                Value::Int(Some(1)),
+                Value::Double(Some(2.4)),
+                Value::String(Some(Box::new("b".to_owned()))),
+                Value::TinyUnsigned(Some(123)),
+                Value::SmallUnsigned(Some(456))
+            )
+        );
+        assert_eq!(
+            (1i32, 2.4f64, "b", 123u8, 456u16, 789u32).into_value_tuple(),
+            ValueTuple::Six(
+                Value::Int(Some(1)),
+                Value::Double(Some(2.4)),
+                Value::String(Some(Box::new("b".to_owned()))),
+                Value::TinyUnsigned(Some(123)),
+                Value::SmallUnsigned(Some(456)),
+                Value::Unsigned(Some(789))
+            )
+        );
     }
 
     #[test]
@@ -885,6 +1279,21 @@ mod tests {
         let original = val.clone();
         val = FromValueTuple::from_value_tuple(val);
         assert_eq!(val, original);
+
+        let mut val = (1i32, 2.4f64, "b".to_owned(), 123u8);
+        let original = val.clone();
+        val = FromValueTuple::from_value_tuple(val);
+        assert_eq!(val, original);
+
+        let mut val = (1i32, 2.4f64, "b".to_owned(), 123u8, 456u16);
+        let original = val.clone();
+        val = FromValueTuple::from_value_tuple(val);
+        assert_eq!(val, original);
+
+        let mut val = (1i32, 2.4f64, "b".to_owned(), 123u8, 456u16, 789u32);
+        let original = val.clone();
+        val = FromValueTuple::from_value_tuple(val);
+        assert_eq!(val, original);
     }
 
     #[test]
@@ -906,6 +1315,43 @@ mod tests {
             Value::String(Some(Box::new("b".to_owned())))
         );
         assert_eq!(iter.next(), None);
+
+        let mut iter = (1i32, 2.4f64, "b", 123u8).into_value_tuple().into_iter();
+        assert_eq!(iter.next().unwrap(), Value::Int(Some(1)));
+        assert_eq!(iter.next().unwrap(), Value::Double(Some(2.4)));
+        assert_eq!(
+            iter.next().unwrap(),
+            Value::String(Some(Box::new("b".to_owned())))
+        );
+        assert_eq!(iter.next().unwrap(), Value::TinyUnsigned(Some(123)));
+        assert_eq!(iter.next(), None);
+
+        let mut iter = (1i32, 2.4f64, "b", 123u8, 456u16)
+            .into_value_tuple()
+            .into_iter();
+        assert_eq!(iter.next().unwrap(), Value::Int(Some(1)));
+        assert_eq!(iter.next().unwrap(), Value::Double(Some(2.4)));
+        assert_eq!(
+            iter.next().unwrap(),
+            Value::String(Some(Box::new("b".to_owned())))
+        );
+        assert_eq!(iter.next().unwrap(), Value::TinyUnsigned(Some(123)));
+        assert_eq!(iter.next().unwrap(), Value::SmallUnsigned(Some(456)));
+        assert_eq!(iter.next(), None);
+
+        let mut iter = (1i32, 2.4f64, "b", 123u8, 456u16, 789u32)
+            .into_value_tuple()
+            .into_iter();
+        assert_eq!(iter.next().unwrap(), Value::Int(Some(1)));
+        assert_eq!(iter.next().unwrap(), Value::Double(Some(2.4)));
+        assert_eq!(
+            iter.next().unwrap(),
+            Value::String(Some(Box::new("b".to_owned())))
+        );
+        assert_eq!(iter.next().unwrap(), Value::TinyUnsigned(Some(123)));
+        assert_eq!(iter.next().unwrap(), Value::SmallUnsigned(Some(456)));
+        assert_eq!(iter.next().unwrap(), Value::Unsigned(Some(789)));
+        assert_eq!(iter.next(), None);
     }
 
     #[test]
@@ -929,6 +1375,27 @@ mod tests {
         assert_eq!(out, timestamp);
     }
 
+    #[test]
+    #[cfg(feature = "with-chrono")]
+    fn test_chrono_utc_value() {
+        let timestamp =
+            DateTime::<Utc>::from_utc(NaiveDate::from_ymd(2022, 1, 2).and_hms(3, 4, 5), Utc);
+        let value: Value = timestamp.into();
+        let out: DateTime<Utc> = value.unwrap();
+        assert_eq!(out, timestamp);
+    }
+
+    #[test]
+    #[cfg(feature = "with-chrono")]
+    fn test_chrono_local_value() {
+        let timestamp_utc =
+            DateTime::<Utc>::from_utc(NaiveDate::from_ymd(2022, 1, 2).and_hms(3, 4, 5), Utc);
+        let timestamp_local: DateTime<Local> = timestamp_utc.into();
+        let value: Value = timestamp_local.into();
+        let out: DateTime<Local> = value.unwrap();
+        assert_eq!(out, timestamp_local);
+    }
+
     #[test]
     #[cfg(feature = "with-chrono")]
     fn test_chrono_timezone_value() {
@@ -984,4 +1451,13 @@ mod tests {
         let out: Decimal = v.unwrap();
         assert_eq!(out.to_string(), num);
     }
+
+    #[test]
+    #[cfg(feature = "postgres-array")]
+    fn test_array_value() {
+        let array = vec![1, 2, 3, 4, 5];
+        let v: Value = array.into();
+        let out: Vec<i32> = v.unwrap();
+        assert_eq!(out, vec![1, 2, 3, 4, 5]);
+    }
 }
